/*         ______   ___    ___
 *        /\  _  \ /\_ \  /\_ \
 *        \ \ \L\ \\//\ \ \//\ \      __     __   _ __   ___ 
 *         \ \  __ \ \ \ \  \ \ \   /'__`\ /'_ `\/\`'__\/ __`\
 *          \ \ \/\ \ \_\ \_ \_\ \_/\  __//\ \L\ \ \ \//\ \L\ \
 *           \ \_\ \_\/\____\/\____\ \____\ \____ \ \_\\ \____/
 *            \/_/\/_/\/____/\/____/\/____/\/___L\ \/_/ \/___/
 *                                           /\____/
 *                                           \_/__/
 *
 *      PSP header defines.
 *
 *      By Julian Smythe.
 *
 *      See readme.txt for copyright information.
 */

#ifndef ALLEGRO_H
	#error must include allegro.h first
#endif /* ALLEGRO_H */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <time.h>
#include <math.h>

#include <unistd.h>
#include <sys/time.h>
#include <sys/stat.h>

#include <pspkernel.h>
#include <pspsdk.h>
#include <pspdebug.h>
#include <pspdisplay.h>
#include <pspctrl.h>
#include <psppower.h>

#include <pspgu.h>
#include <pspgum.h>
#include <pspge.h>

#include <pspiofilemgr.h>
#include <pspsystimer.h>

#define ALLEGRO_PLATFORM_STR "PSP"
#define ALLEGRO_HAVE_STDINT_H 1
#define ALLEGRO_LITTLE_ENDIAN 1

#define PSP_ALIGNED(w) __attribute__((aligned(w)))

/* #define PSP_ALIGN(w,n) ((int)(n+w-1)/w*w) */

#define PSP_A_SCREEN_W (512)
#define PSP_SCREEN_W (480)
#define PSP_SCREEN_H (272)
#define PSP_VRAM_BASE (0x40000000)
#define PSP_VRAM_SIZE (2*1024*1024)

#define PSP_VRAM(t,x) ((void *)(PSP_VRAM_BASE+(t)x))

#ifndef ALLEGRO_PSP
	#error bad include
#endif /* ALLEGRO_PSP */

// Why the hell is dup() missing
#define dup __psp_fdman_get_dup_descriptor
int __psp_fdman_get_dup_descriptor(int fd);


#define PSP_BEGIN_DRIVER_LIST(x) _DRIVER_INFO _##x##_driver_list[] = {

#define PSP_END_DRIVER_LIST {0, NULL, 0}};



/*******************************************/
/************* system drivers **************/
/*******************************************/
#define SYSTEM_PSP           AL_ID('S','P','S','P')

AL_VAR(SYSTEM_DRIVER, systemdrv_psp);

#define SYSTEM_DRIVER_PSP { SYSTEM_PSP, &systemdrv_psp, TRUE },


/*******************************************/
/************** timer drivers **************/
/*******************************************/
#define TIMER_PSP    AL_ID('T','P','S','P')

AL_VAR(TIMER_DRIVER, timedrv_psp);

#define TIMER_DRIVER_PSP { TIMER_PSP, &timedrv_psp, TRUE },


/*******************************************/
/************ keyboard drivers *************/
/*******************************************/
#define KEYBOARD_PSP_IR         AL_ID('K','P','S','I')
/*#define KEYBOARD_PSP_SIO        AL_ID('K','P','S','S')*/

AL_VAR(KEYBOARD_DRIVER, keydrv_psp_ir);
/*AL_VAR(KEYBOARD_DRIVER, keydrv_psp_sio);*/

#define KEYBOARD_DRIVER_PSP_IR { KEYBOARD_PSP_IR, &keydrv_psp_ir, TRUE },
//#define KEYBOARD_DRIVER_PSP_SIO { KEYBOARD_PSP_SIO, &keydrv_psp_sio, FALSE },


/*******************************************/
/************* mouse drivers ***************/
/*******************************************/
#define MOUSE_PSP_ANALOG     AL_ID('M','P','S','A')

AL_VAR(MOUSE_DRIVER, mousedrv_psp_analog);

#define MOUSE_DRIVER_PSP_ANALOG { MOUSE_PSP_ANALOG, &mousedrv_psp_analog, TRUE },


/*******************************************/
/*************** gfx drivers ***************/
/*******************************************/
#define GFX_PSP                  AL_ID('G','P','S','P')

AL_VAR(GFX_DRIVER, gfxdrv_psp);

#define GFX_DRIVER_PSP { GFX_PSP, &gfxdrv_psp, TRUE },



/********************************************/
/*************** sound drivers **************/
/********************************************/
/* TODO: Sound */


/*******************************************/
/************ joystick drivers *************/
/*******************************************/
#define JOYSTICK_PSP         AL_ID('J','P','S','P')

AL_VAR(JOYSTICK_DRIVER, joystickdrv_psp);

#define JOYSTICK_DRIVER_PSP { JOYSTICK_PSP, &joystickdrv_psp, TRUE },


#ifdef TRACE
#undef TRACE
#endif

#define TRACE allegro_message
