#ifndef __PSP_COMMON_H_
#define __PSP_COMMON_H_

#include "allegro.h"
#include "allegro/internal/aintern.h"

/*
CBCD = Call by color depth
CBCDA = Call by color depth (with arguments)
CBCDR = Call by color depth and return
CBCDAR = Call by color depth and return (with arguments)
*/

#define AL_PSP_CBCD(depth,func) \
switch (depth) \
{ \
	case 8: gfx_psp_8_##func(); break; \
	case 16: gfx_psp_16_##func(); break; \
	case 24: gfx_psp_24_##func(); break; \
	default: case 32: gfx_psp_32_##func(); break; \
}

#define AL_PSP_CBCDA(depth,func,...) \
switch (depth) \
{ \
	case 8: gfx_psp_8_##func(__VA_ARGS__); break; \
	case 16: gfx_psp_16_##func(__VA_ARGS__); break; \
	case 24: gfx_psp_24_##func(__VA_ARGS__); break; \
	default: case 32: gfx_psp_32_##func(__VA_ARGS__); break; \
}

#define AL_PSP_CBCDR(depth,func) \
switch (depth) \
{ \
	case 8: return gfx_psp_8_##func(); \
	case 16: return gfx_psp_16_##func(); \
	case 24: return gfx_psp_24_##func(); \
	default: case 32: return gfx_psp_32_##func(); \
}

#define AL_PSP_CBCDAR(depth,func,...) \
switch (depth) \
{ \
	case 8: return gfx_psp_8_##func(__VA_ARGS__); \
	case 16: return gfx_psp_16_##func(__VA_ARGS__); \
	case 24: return gfx_psp_24_##func(__VA_ARGS__); \
	default: case 32: return gfx_psp_32_##func(__VA_ARGS__); \
}

#define COUNTER_TO_MSEC(x) ((unsigned long)(x / counter_per_msec.QuadPart))
#define COUNTER_TO_TIMER(x) ((unsigned long)(x * TIMERS_PER_SECOND / counter_freq.QuadPart))
#define MSEC_TO_COUNTER(x) (x * counter_per_msec.QuadPart)
#define TIMER_TO_COUNTER(x) (x * counter_freq.QuadPart / TIMERS_PER_SECOND)
#define TIMER_TO_MSEC(x) ((unsigned long)(x) / (TIMERS_PER_SECOND / 1000))

#endif /* __PSP_COMMON_H_ */
