/*         ______   ___    ___
 *        /\  _  \ /\_ \  /\_ \
 *        \ \ \L\ \\//\ \ \//\ \      __     __   _ __   ___ 
 *         \ \  __ \ \ \ \  \ \ \   /'__`\ /'_ `\/\`'__\/ __`\
 *          \ \ \/\ \ \_\ \_ \_\ \_/\  __//\ \L\ \ \ \//\ \L\ \
 *           \ \_\ \_\/\____\/\____\ \____\ \____ \ \_\\ \____/
 *            \/_/\/_/\/____/\/____/\/____/\/___L\ \/_/ \/___/
 *                                           /\____/
 *                                           \_/__/
 *
 *      Joystick driver
 *
 *      By Julian Smythe.
 *
 *      See readme.txt for copyright information.
 */

#include "pcommon.h"

static SceCtrlData joystick_psp_ctrldata;

int joystick_psp_init(void);
void joystick_psp_exit(void);
int joystick_psp_poll(void);
int joystick_psp_save_data(void);
int joystick_psp_load_data(void);
AL_CONST char *joystick_psp_calibrate_name(int n);
int joystick_psp_calibrate(int n);

JOYSTICK_DRIVER joystickdrv_psp =
{
	JOYSTICK_PSP,
	"JOYSTICK_PSP",
	"PSP Joystick Driver",
	"JOYSTICK_PSP",
	joystick_psp_init,
	joystick_psp_exit,
	joystick_psp_poll,
	NULL,//joystick_psp_save_data,
	NULL,//joystick_psp_load_data,
	NULL,//joystick_psp_calibrate_name,
	NULL //joystick_psp_calibrate
};

int joystick_psp_init(void)
{
	sceCtrlSetSamplingCycle(0);
	sceCtrlSetSamplingMode(PSP_CTRL_MODE_DIGITAL);
	sceCtrlReadBufferPositive(&joystick_psp_ctrldata,1);
	return 0;
}

void joystick_psp_exit(void)
{

}

int joystick_psp_poll(void)
{
	sceCtrlSetSamplingMode(PSP_CTRL_MODE_DIGITAL);
	sceCtrlReadBufferPositive(&joystick_psp_ctrldata,1);
	return 0;
}
