/*         ______   ___    ___
 *        /\  _  \ /\_ \  /\_ \
 *        \ \ \L\ \\//\ \ \//\ \      __     __   _ __   ___ 
 *         \ \  __ \ \ \ \  \ \ \   /'__`\ /'_ `\/\`'__\/ __`\
 *          \ \ \/\ \ \_\ \_ \_\ \_/\  __//\ \L\ \ \ \//\ \L\ \
 *           \ \_\ \_\/\____\/\____\ \____\ \____ \ \_\\ \____/
 *            \/_/\/_/\/____/\/____/\/____/\/___L\ \/_/ \/___/
 *                                           /\____/
 *                                           \_/__/
 *
 *      IR keyboard driver
 *
 *      By Julian Smythe.
 *
 *      See readme.txt for copyright information.
 */

#include "pkbd_ir.h"
#include "pcommon.h"

int keyboard_psp_ir_keys = 332;
int keyboard_psp_ir_hard_keys = 511; /* 0x1ff */

static const char *keyboard_psp_ir_prx = "ms0:/seplugins/irkeyb.prx";

AL_CONST unsigned char keyboard_psp_ir_convert[] = {
	/*   0 */ 0,             KEY_ESC,       KEY_1,         KEY_2,
	/*   4 */ KEY_3,         KEY_4,         KEY_5,         KEY_6,
	/*   8 */ KEY_7,         KEY_8,         KEY_9,         KEY_0,
	/*  12 */ KEY_MINUS,     KEY_EQUALS,    KEY_BACKSPACE, KEY_TAB,
	/*  16 */ KEY_Q,         KEY_W,         KEY_E,         KEY_R,
	/*  20 */ KEY_T,         KEY_Y,         KEY_U,         KEY_I,
	/*  24 */ KEY_O,         KEY_P,         KEY_OPENBRACE, KEY_CLOSEBRACE,
	/*  28 */ KEY_ENTER,     KEY_LCONTROL,  KEY_A,         KEY_S,
	/*  32 */ KEY_D,         KEY_F,         KEY_G,         KEY_H,
	/*  36 */ KEY_J,         KEY_K,         KEY_L,         KEY_SEMICOLON,
	/*  40 */ KEY_QUOTE,     KEY_TILDE,     KEY_LSHIFT,    KEY_BACKSLASH,
	/*  44 */ KEY_Z,         KEY_X,         KEY_C,         KEY_V,
	/*  48 */ KEY_B,         KEY_N,         KEY_M,         KEY_COMMA,
	/*  52 */ KEY_STOP,      KEY_SLASH,     KEY_RSHIFT,    KEY_ASTERISK,
	/*  56 */ KEY_ALT,       KEY_SPACE,     KEY_CAPSLOCK,  KEY_F1,
	/*  60 */ KEY_F2,        KEY_F3,        KEY_F4,        KEY_F5,
	/*  64 */ KEY_F6,        KEY_F7,        KEY_F8,        KEY_F9,
	/*  68 */ KEY_F10,       KEY_NUMLOCK,   KEY_SCRLOCK,   KEY_7_PAD,
	/*  72 */ KEY_8_PAD,     KEY_9_PAD,     KEY_MINUS_PAD, KEY_4_PAD,
	/*  76 */ KEY_5_PAD,     KEY_6_PAD,     KEY_PLUS_PAD,  KEY_1_PAD,
	/*  80 */ KEY_2_PAD,     KEY_3_PAD,     KEY_0_PAD,     KEY_STOP,
	/*  84 */ 0,             0,             0,             KEY_F11,
	/*  88 */ KEY_F12,       0,             0,             0,
	/*  92 */ 0,             0,             0,             0,
	/*  96 */ KEY_ENTER,     KEY_RCONTROL,  KEY_SLASH_PAD, KEY_PRTSCR,
	/* 100 */ KEY_ALTGR,     KEY_ENTER,     KEY_HOME,      KEY_UP,
	/* 104 */ KEY_PGUP,      KEY_LEFT,      KEY_RIGHT,     KEY_END,
	/* 108 */ KEY_DOWN,      KEY_PGDN,      KEY_INSERT,    KEY_DEL,
	/* 112 */ 0,             0,             0,             0,
	/* 116 */ 0,             KEY_EQUALS_PAD,0,             KEY_PAUSE,
	/* 120 */ 0,             0,             0,             0,
	/* 124 */ KEY_COMMA,     0,             0,             0,
	/* 128 */ 0,             0,             0,             KEY_Z,
	/* 132 */ 0,             KEY_C,         KEY_O,         KEY_V,
	/* 136 */ KEY_F,         KEY_X,         KEY_F1,        KEY_ESC,
	/* 140 */ 0,             0,             0,             0,
	/* 144 */ 0,             0,             0,             0,
	/* 148 */ 0,             0,             0,             0,
	/* 152 */ 0,             0,             0,             0,
	/* 156 */ 0,             0,             KEY_LEFT,      KEY_RIGHT,
	/* 160 */ 0,             0,             0,             KEY_RIGHT,
	/* 164 */ 0,             KEY_LEFT,      0,             0,
	/* 168 */ KEY_LEFT,      0,             0,             0,
	/* 172 */ 0,             KEY_F5,        KEY_ESC,       0,
	/* 176 */ 0,             KEY_PGUP,      KEY_PGDN,      KEY_9,
	/* 180 */ KEY_0,         0,             0,             0,
	/* 184 */ 0,             0,             0,             0,
	/* 188 */ 0,             0,             0,             0,
	/* 192 */ 0,             0,             0,             0,
	/* 196 */ 0,             0,             0,             0,
	/* 200 */ 0,             0,             0,             0,
	/* 204 */ 0,             0,             0,             0,
	/* 208 */ 0,             0,             0,             0,
	/* 212 */ 0,             0,             0,             0,
	/* 216 */ 0,             0,             0,             0,
	/* 220 */ 0,             0,             0,             0,
	/* 224 */ 0,             0,             0,             0,
	/* 228 */ 0,             0,             0,             0,
	/* 232 */ 0,             0,             0,             0,
	/* 236 */ 0,             0,             0,             0,
	/* 240 */ 0,             0,             0,             0,
	/* 244 */ 0,             0,             0,             0,
	/* 248 */ 0,             0,             0,             0,
	/* 252 */ 0,             0,             0,             0,
	/* 256 */ 0,             KEY_0,         KEY_1,         KEY_2,
	/* 260 */ KEY_3,         KEY_4,         KEY_5,         KEY_6,
	/* 264 */ KEY_7,         KEY_8,         KEY_9,         0,
	/* 268 */ 0,             0,             0,             0,
	/* 272 */ 0,             0,             0,             0,
	/* 276 */ 0,             0,             0,             0,
	/* 280 */ 0,             0,             0,             0,
	/* 284 */ 0,             0,             0,             0,
	/* 288 */ 0,             0,             0,             0,
	/* 292 */ 0,             0,             0,             0,
	/* 296 */ 0,             0,             0,             0,
	/* 300 */ 0,             0,             0,             0,
	/* 304 */ 0,             KEY_A,         KEY_B,         KEY_C,
	/* 308 */ KEY_X,         KEY_Y,         KEY_Z,         KEY_L,
	/* 312 */ KEY_R,         KEY_L,         KEY_R,         0,
	/* 316 */ KEY_ENTER,     0,             0,             0,
	/* 320 */ 0,             0,
	/* Pad to 511 just in case */
	0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0, /* 350 */
	0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0, /* 380 */
	0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0, /* 410 */
	0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0, /* 440 */
	0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0, /* 470 */
	0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0, /* 500 */
	0,0,0,0,0,0,0,0,0,0,0 /* 511 */
};

static SceUID keyboard_psp_ir_thread_id;

int keyboard_psp_ir_thread(SceSize args, void *argp);

int keyboard_psp_ir_init(void);
void keyboard_psp_ir_exit(void);
void keyboard_psp_ir_poll(void);
void keyboard_psp_ir_set_leds(int leds);
void keyboard_psp_ir_set_rate(int delay, int rate);
void keyboard_psp_ir_wait_for_input(void);
void keyboard_psp_ir_stop_waiting_for_input(void);
int keyboard_psp_ir_scancode_to_ascii(int scancode);
AL_CONST char *keyboard_psp_ir_scancode_to_name(int scancode);

KEYBOARD_DRIVER keydrv_psp_ir =
{
	KEYBOARD_PSP_IR,
	"KEYBOARD_PSP_IR",
	"PSP IR Keyboard Driver",
	"KEYBOARD_PSP_IR",
	TRUE,
	keyboard_psp_ir_init,
	keyboard_psp_ir_exit,
	NULL,//keyboard_psp_ir_poll
	NULL,//keyboard_psp_ir_set_leds
	NULL,//keyboard_psp_ir_set_rate,
	NULL,//keyboard_psp_ir_wait_for_input,
	NULL,//keyboard_psp_ir_stop_waiting_for_input,
	NULL,//keyboard_psp_ir_scancode_to_ascii,
	NULL //keyboard_psp_ir_scancode_to_name,
};

int keyboard_psp_ir_init(void)
{
	TRACE("Initializing IR keyboard driver\n");
	int ret = pspIrKeybInit(NULL,0);
	if (ret == PSP_IRKBD_RESULT_OK)
	{
		pspIrKeybOutputMode(PSP_IRKBD_OUTPUT_MODE_SCANCODE);
		keyboard_psp_ir_thread_id = sceKernelCreateThread(keydrv_psp_ir.desc,keyboard_psp_ir_thread,0x02,0x7FFF,0,NULL);
		if (keyboard_psp_ir_thread_id >= 0)
		{
			sceKernelStartThread(keyboard_psp_ir_thread_id,0,0);
		}
		else
		{
			TRACE("Initializing IR keyboard driver FAILED (1:%x)\n",keyboard_psp_ir_thread_id);
			pspIrKeybFinish();
			return 1;
		}
		return 0;
	}
	else
	{
		TRACE("Initializing IR keyboard driver FAILED (2:%x)\n",ret);
		return 1;
	}
}

void keyboard_psp_ir_exit(void)
{
	TRACE("Terminating thread\n");
	sceKernelTerminateDeleteThread(keyboard_psp_ir_thread_id);
	TRACE("Removing IR keyboard driver\n");
	pspIrKeybFinish();
}

int keyboard_psp_ir_thread(SceSize args, void *argp)
{
    unsigned char shift, ctrl, alt, pressed, raw, translated;
	unsigned char buffer[1024];
	int ret, i, length, count, shifts;
	SIrKeybScanCodeData * scandata;

	while(1)
	{
		if ((ret = pspIrKeybReadinput(buffer,&length)) >= 0)
		{
			if (length % sizeof(SIrKeybScanCodeData) > 0)
			{
				TRACE(" IR error: read length is %d\n",length);
			}
			count = length / sizeof(SIrKeybScanCodeData);
			for (i = 0; i < count; ++i)
			{
				scandata = (SIrKeybScanCodeData *)buffer+i;
				raw = scandata->raw;
				shift = scandata->shift;
				ctrl = scandata->ctrl;
				alt = scandata->alt;
				pressed = scandata->pressed;
				
				if (raw == _KIR_RESERVED)
				{
					TRACE(" IR error: reserved key\n");
				}
				else if (raw >= keyboard_psp_ir_hard_keys)
				{
					TRACE(" IR error: key > KEY_MAX (%d)\n",keyboard_psp_ir_hard_keys);
				}
				else
				{
					translated = keyboard_psp_ir_convert[raw];
					if (translated == 0)
					{
						TRACE(" IR warning: No KEY_ constant mapped to %d (%s)\n",raw,pspIrKeybRawkeyName(raw));
					}
					else
					{
						if (pressed)
						{
							_handle_key_press(scancode_to_ascii(translated),translated);
						}
						else
						{
							_handle_key_release(translated);
						}
					}
				}
			}
		}
		else
		{
			sceKernelDelayThread(10000);
		}
	}

	sceKernelExitThread(0);
	return 0;
}
