/*         ______   ___    ___
 *        /\  _  \ /\_ \  /\_ \
 *        \ \ \L\ \\//\ \ \//\ \      __     __   _ __   ___ 
 *         \ \  __ \ \ \ \  \ \ \   /'__`\ /'_ `\/\`'__\/ __`\
 *          \ \ \/\ \ \_\ \_ \_\ \_/\  __//\ \L\ \ \ \//\ \L\ \
 *           \ \_\ \_\/\____\/\____\ \____\ \____ \ \_\\ \____/
 *            \/_/\/_/\/____/\/____/\/____/\/___L\ \/_/ \/___/
 *                                           /\____/
 *                                           \_/__/
 *
 *      Analog stick mouse driver
 *
 *      By Julian Smythe.
 *
 *      See readme.txt for copyright information.
 */

#include "pcommon.h"

static SceCtrlData mouse_psp_analog_ctrldata;
static SceUID mouse_psp_analog_thread_id;

int mouse_psp_analog_thread(SceSize args, void *argp);

int mouse_psp_analog_init(void);
void mouse_psp_analog_exit(void);
void mouse_psp_analog_poll(void);
void mouse_psp_analog_timer_poll(void);
void mouse_psp_analog_position(int x, int y);
void mouse_psp_analog_set_range(int x1, int y_1, int x2, int y2);
void mouse_psp_analog_set_speed(int xspeed, int yspeed);
void mouse_psp_analog_get_mickeys(int *mickeyx, int *mickeyy);
void mouse_psp_analog_analyze_data(AL_CONST char *buffer, int size);
void mouse_psp_analog_enable_hardware_cursor(int mode);
void mouse_psp_analog_select_system_cursor(int cursor);

MOUSE_DRIVER mousedrv_psp_analog =
{
	MOUSE_PSP_ANALOG,
	"MOUSE_PSP_ANALOG",
	"PSP Analog Stick Mouse Driver",
	"MOUSE_PSP_ANALOG",
	mouse_psp_analog_init,
	mouse_psp_analog_exit,
	mouse_psp_analog_poll,
	mouse_psp_analog_timer_poll,
	NULL,//mouse_psp_analog_position,
	NULL,//mouse_psp_analog_set_range,
	NULL,//mouse_psp_analog_set_speed,
	NULL,//mouse_psp_analog_get_mickeys,
	NULL,//mouse_psp_analog_analyze_data,
	NULL,//mouse_psp_analog_enable_hardware_cursor,
	NULL //mouse_psp_analog_select_system_cursor
};

int mouse_psp_analog_init(void)
{
	TRACE("Initializing mouse system...\n");
	mouse_x = PSP_SCREEN_W/2;
	mouse_y = PSP_SCREEN_H/2;
	mouse_psp_analog_thread_id = sceKernelCreateThread(mousedrv_psp_analog.desc,mouse_psp_analog_thread,0x02,0x7FFF,0,NULL);
	if (mouse_psp_analog_thread_id >= 0)
	{
		sceKernelStartThread(mouse_psp_analog_thread_id,0,0);
	}
	else
	{
		TRACE("Initializing mouse system FAILED... (%x)\n",mouse_psp_analog_thread_id);
		return 1;
	}
	return 0;
}

void mouse_psp_analog_exit(void)
{
	sceKernelTerminateDeleteThread(mouse_psp_analog_thread_id);
}

void mouse_psp_analog_poll(void)
{
	sceCtrlSetSamplingMode(PSP_CTRL_MODE_ANALOG);
	sceCtrlReadBufferPositive(&mouse_psp_analog_ctrldata,1);
	mouse_x += (mouse_psp_analog_ctrldata.Lx-128)/40;
	mouse_y += (mouse_psp_analog_ctrldata.Ly-128)/40;
	if (mouse_x < 0) mouse_x = 0;
	if (mouse_x >= PSP_SCREEN_W) mouse_x = PSP_SCREEN_W-1;

	if (mouse_y < 0) mouse_y = 0;
	if (mouse_y >= PSP_SCREEN_H) mouse_y = PSP_SCREEN_H-1;

	mouse_b = 0;
	if ((mouse_psp_analog_ctrldata.Buttons & PSP_CTRL_CROSS) == PSP_CTRL_CROSS)
	{
		mouse_b |= 1;
	}
	if ((mouse_psp_analog_ctrldata.Buttons & PSP_CTRL_CIRCLE) == PSP_CTRL_CIRCLE)
	{
		mouse_b |= 2;
	}
	if ((mouse_psp_analog_ctrldata.Buttons & PSP_CTRL_TRIANGLE) == PSP_CTRL_TRIANGLE)
	{
		mouse_b |= 4;
	}

	_handle_mouse_input();
}

void mouse_psp_analog_timer_poll(void)
{
	mouse_psp_analog_poll();
}

int mouse_psp_analog_thread(SceSize args, void *argp)
{
	while(1)
	{
		mouse_psp_analog_poll();

		sceKernelDelayThread(25000);
	}

	sceKernelExitThread(0);
	return 0;
}