/*         ______   ___    ___
 *        /\  _  \ /\_ \  /\_ \
 *        \ \ \L\ \\//\ \ \//\ \      __     __   _ __   ___ 
 *         \ \  __ \ \ \ \  \ \ \   /'__`\ /'_ `\/\`'__\/ __`\
 *          \ \ \/\ \ \_\ \_ \_\ \_/\  __//\ \L\ \ \ \//\ \L\ \
 *           \ \_\ \_\/\____\/\____\ \____\ \____ \ \_\\ \____/
 *            \/_/\/_/\/____/\/____/\/____/\/___L\ \/_/ \/___/
 *                                           /\____/
 *                                           \_/__/
 *
 *      Allegro core driver
 *
 *      By Julian Smythe.
 *
 *      See readme.txt for copyright information.
 */

#include "pcommon.h"

PSP_MODULE_INFO("Allegro", 0x1000, 1, 1);
PSP_MAIN_THREAD_ATTR(0);

int sys_psp_init(void);
void sys_psp_exit(void);
void sys_psp_get_executable_name(char *output, int size);
void sys_psp_set_window_title(AL_CONST char *name);
void sys_psp_message(AL_CONST char *msg);

SYSTEM_DRIVER systemdrv_psp =
{
	SYSTEM_PSP,
	"SYSTEM_PSP",
	"PSP System Driver",
	"SYSTEM_PSP",
	sys_psp_init,
	sys_psp_exit,
	NULL,//sys_psp_get_executable_name,
	NULL,//sys_psp_find_resource
	NULL,//sys_psp_set_window_title,
	NULL,//sys_psp_set_close_button_callback,
	sys_psp_message,
	NULL,//sys_psp_assert,
	NULL,//sys_psp_save_console_state,
	NULL,//sys_psp_restore_console_state,
	NULL,//sys_psp_create_bitmap,
	NULL,//sys_psp_created_bitmap,
	NULL,//sys_psp_create_sub_bitmap,
	NULL,//sys_psp_created_sub_bitmap,
	NULL,//sys_psp_destroy_bitmap,
	NULL,//sys_psp_read_hardware_palette,
	NULL,//sys_psp_set_palette_range,
	NULL,//sys_psp_get_vtable,
	NULL,//sys_psp_set_display_switch_mode,
	NULL,//sys_psp_display_switch_lock,
	NULL,//sys_psp_desktop_color_depth,
	NULL,//sys_psp_get_desktop_resolution,
	NULL,//sys_psp_get_gfx_safe_mode,
	NULL,//sys_psp_yield_timeslice,
	NULL,//sys_psp_create_mutex,
	NULL,//sys_psp_destroy_mutex,
	NULL,//sys_psp_lock_mutex,
	NULL,//sys_psp_unlock_mutex,
	NULL,//sys_psp_gfx_drivers,
	NULL,//sys_psp_digi_drivers,
	NULL,//sys_psp_midi_drivers,
	NULL,//sys_psp_keyboard_drivers,
	NULL,//sys_psp_mouse_drivers,
	NULL,//sys_psp_joystick_drivers,
	NULL //sys_psp_timer_drivers
};

int sys_psp_exit_callback(int arg1, int arg2, void *common)
{
	/*TODO: Hook this to the close button callback or smth. */
	allegro_exit();
	return 0;
}

int sys_psp_callback_thread(SceSize args, void *argp)
{
	int cbid;

	cbid = sceKernelCreateCallback("Exit Callback", sys_psp_exit_callback, NULL);
	sceKernelRegisterExitCallback(cbid);

	sceKernelSleepThreadCB();

	return 0;
}


int sys_psp_setup_callbacks(void)
{
	int thid = 0;

	thid = sceKernelCreateThread("update_thread", sys_psp_callback_thread, 0x10, 0xFA0, 0, 0);
	if(thid >= 0)
	{
		sceKernelStartThread(thid, 0, 0);
	}

	return thid;
}


int sys_psp_init(void)
{
	sys_psp_setup_callbacks();
	pspDebugScreenInit();
	sceGuInit();
	return 0;
}

void sys_psp_exit(void)
{
	sceGuTerm();
	sceKernelExitGame();
}

void sys_psp_message(AL_CONST char *msg)
{
	pspDebugScreenPuts(msg);
	sceKernelDelayThread(800000);
}
