/*         ______   ___    ___
 *        /\  _  \ /\_ \  /\_ \
 *        \ \ \L\ \\//\ \ \//\ \      __     __   _ __   ___ 
 *         \ \  __ \ \ \ \  \ \ \   /'__`\ /'_ `\/\`'__\/ __`\
 *          \ \ \/\ \ \_\ \_ \_\ \_/\  __//\ \L\ \ \ \//\ \L\ \
 *           \ \_\ \_\/\____\/\____\ \____\ \____ \ \_\\ \____/
 *            \/_/\/_/\/____/\/____/\/____/\/___L\ \/_/ \/___/
 *                                           /\____/
 *                                           \_/__/
 *
 *      Timer driver
 *
 *      By Julian Smythe.
 *
 *      See readme.txt for copyright information.
 */

#include "pcommon.h"

static SceUID timer_psp_thread_id;

int timer_psp_thread(SceSize args, void *argp);

int timer_psp_init(void);
void timer_psp_exit(void);
void timer_psp_install_int(AL_METHOD(void, proc, (void)), long speed);
void timer_psp_remove_int(AL_METHOD(void, proc, (void)));
int timer_psp_install_param_int(AL_METHOD(void, proc, (void *param)), void *param, long speed);
void timer_psp_remove_param_int(AL_METHOD(void, proc, (void *param)), void *param);
int timer_psp_can_simulate_retrace(void);
void timer_psp_simulate_retrace(int enable);
void timer_psp_rest(unsigned int tyme, AL_METHOD(void, callback, (void)));

static AL_METHOD(void, timer_psp_callback, (void));

int timer_psp_callback_wrapper(void)
{
	timer_psp_callback();
	return -1;
}

TIMER_DRIVER timedrv_psp =
{
	TIMER_PSP,
	"TIMER_PSP",
	"PSP Timing Driver",
	"TIMER_PSP",
	timer_psp_init,
	timer_psp_exit,
	NULL,//timer_psp_install_int,
	NULL,//timer_psp_remove_int,
	NULL,//timer_psp_install_param_int,
	NULL,//timer_psp_remove_param_int,
	NULL,//timer_psp_can_simulate_retrace,
	NULL,//timer_psp_simulate_retrace,
	timer_psp_rest
};

int timer_psp_init(void)
{
	TRACE("Initializing timer system...\n");
	timer_psp_thread_id = sceKernelCreateThread(timedrv_psp.desc,timer_psp_thread,0x11,0x7FFF,0,NULL);
	if (timer_psp_thread_id >= 0)
	{
		sceKernelStartThread(timer_psp_thread_id,0,0);
	}
	else
	{
		TRACE("Initializing timer system FAILED (%x)...\n",timer_psp_thread_id);
		return 1;
	}
	return 0;
}

void timer_psp_exit(void)
{
	TRACE("Removing timer system...\n");
	sceKernelTerminateDeleteThread(timer_psp_thread_id);
}


void timer_psp_rest(unsigned int tyme, AL_METHOD(void, callback, (void)))
{
	sceKernelDelayThread(tyme*1000);
}

int timer_psp_thread(SceSize args, void *argp)
{
	unsigned long long int prev_time;
	unsigned long long int curr_time;
	unsigned long long int diff_time;
	const unsigned long int clocks_per_msec = CLOCKS_PER_SEC/1000;
	
	if (clocks_per_msec == 0)
	{
		TRACE("Resolution of clock() is too low\n");
	}
	
	prev_time = clock() / clocks_per_msec;
	
	if (prev_time == 0)
	{
		TRACE("clock() b0rked\n");
	}
	
	while(1)
	{
		curr_time = clock() / clocks_per_msec;
		diff_time = curr_time - prev_time;
		prev_time = curr_time;
	
		sceKernelDelayThread(TIMER_TO_MSEC(_handle_timer_tick(MSEC_TO_TIMER(diff_time)))*1000);
	}

	sceKernelExitThread(0);
	return 0;
}
